# -*- python -*-
# $RCSfile: oof_mainiteration.py,v $
# $Revision: 1.27 $
# $Author: reida $
# $Date: 2006/08/23 23:56:06 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import threadstate
from oof2.common import debug
from oof2.common import mainthread
from oof2.common import thread_enable
import threading
import gtk
import gobject

_inside_idle_callback = 0


# Thread-safe GTK mainiteration call -- wraps the gtk.mainiteration
# in threads_enter and threads_leave in the appropriate circumstances,
# i.e. when the main thread is in an idle-callback, or when threads
# are enabled and the caller is not the main thread.  This is
# required because both subthreads and idle_callbacks need to acquire
# the GTK lock before making GTK calls.
def mainiteration(block=True):
     global _inside_idle_callback
     main = mainthread.mainthread()
     needgtklock = (thread_enable.query() and not main) or \
                   (_inside_idle_callback and main)
     if needgtklock:
##          debug.fmsg("mainiteration gtk.threads_enter()")
          gtk.gdk.threads_enter()

     try:
          gtk.main_iteration(block)
     finally:
          if needgtklock:
               gtk.gdk.threads_leave()
##               debug.fmsg("mainiteration gtk.threads_leave()")

# Looping version, processes the full event queue.  This also
# wraps the events_pending call, which is necessary to prevent
# blocking.
def mainiteration_loop(block=True):
     global _inside_idle_callback
     main = mainthread.mainthread()
     needgtklock = (thread_enable.query() and not main) or \
                   (_inside_idle_callback and main)
     if needgtklock:
##          debug.fmsg("mainiteration_loop gtk.threads_enter()")
          gtk.threads_enter()
     while gtk.events_pending():
          gtk.main_iteration(block)
     if needgtklock:
          gtk.threads_leave()
##          debug.fmsg("mainiteration_loop gtk.threads_leave()")

# The actual wrapped idle callback.  Mainly, this object exists to set
# the _inside_idle_callback flag.

class OOFIdleCallback:
     def __init__(self, func, args=(), kwargs={}):
          self.func = func
          self.args = args
          self.kwargs = kwargs
     def __call__(self):
          global _inside_idle_callback
          _inside_idle_callback = 1
          try:
               self.func(*self.args, **self.kwargs)
          finally:
               _inside_idle_callback = 0
          return False                  # don't repeat

# Adds a wrapped idle-function callback.
def oof_idle_add(func, args=(), kwargs={}):
     gobject.idle_add(OOFIdleCallback(func, args, kwargs))

###########################
     
class OOFIdleBlockCallback(OOFIdleCallback):
     def __init__(self, func, args=(), kwargs={}):
          OOFIdleCallback.__init__(self, func, args, kwargs)
          self.event = threading.Event()
          self.result = None
          self.callingthread = threadstate.findThreadNumber()
     def __call__(self):
          global _inside_idle_callback
          _inside_idle_callback = 1
          try:
##               debug.fmsg('Running blocking function from thread',
##                          self.callingthread)
               gtk.gdk.threads_enter()
               self.result = self.func(*self.args, **self.kwargs)
               gtk.gdk.threads_leave()
##               debug.fmsg('done running blocking function from thread', self.callingthread)
          finally:
               _inside_idle_callback = 0
               self.event.set()
          return False              # don't repeat

def oof_idle_block_add(func, args=(), kwargs={}):
##     debug.fmsg('adding idle callback on thread', threadstate.findThreadNumber())
##     debug.dumpCaller(1)
     callbackobj = OOFIdleBlockCallback(func, args, kwargs)
     callbackobj.event.clear()
##     debug.fmsg('waiting for', func)
     gobject.idle_add(callbackobj, priority=300)
##     debug.fmsg('waiting')
     callbackobj.event.wait()
     # debug.fmsg('done')
     return callbackobj.result
          
