# -*- python -*-
# $RCSfile: threadmanager.py,v $
# $Revision: 1.25 $
# $Author: reida $
# $Date: 2006/04/28 19:02:36 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import string
from oof2.SWIG.common import lock
from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import thread_enable
from oof2.common.IO import reporter


## ThreadManager coordinates the thread-enabled menu items of OOF.
## It administrates the existing running threaded processes (ProgressObjects),
## as well as those threads that have already finished and are waiting to be
## dismissed. It also sends/receives instructions for automatically stoping
## them when the appropriate flag is set.

class ThreadManager:
    def __init__(self):
        self.AutoDismiss = 1       
        self.listofprogressobjects = []
        self.listofworkers = []
        ## Keep list of workers in order to quit properly.
        ## At exit time, the ThreadManager loops over the
        ## running workers and forces them to stop and
        ## join. 
        self.lock = lock.SLock() 
        
    def WaitForAllThreads(self):
        ## called at quitting time by main thread
        ## debug.fmsg("WaitForAllThreads")
        self.lock.acquire()
        try:
            for worker in self.listofworkers :
                worker.join()
        finally:
            self.lock.release()
            
    def quit(self, menuitem = None):
        ## debug.fmsg("quit")
        ## locks put inside each function
        self.stopAll()                  ## tell the threads politely to stop
        ## self.cancelAll()                ## tell remaining threads IMpolitely to stop
        
        ## TODO LATER:
        ## Fix the comment below.
        ##
        ## Strictly speaking all threads MUST stop before the
        ## main thread exits. A better implementation would use
        ## cancelAll(); however, this functionality has a bug
        ## for Macs, and hangs on its way out.
        ## The above line should be un-commented out when (if it ever)
        ## the pthread code works again in Macs
        

    def disableAutoDismiss(self, menuitem, value):
        ## debug.fmsg("disableAutoDismiss")
        self.lock.acquire()
        try:
            self.AutoDismiss = value
        finally:
            self.lock.release()
    def enableThreadingCB(self, menuitem, value):
        ## debug.fmsg("enableThreadingCB")
        self.lock.acquire()
        try:
            thread_enable.set(value)
        finally:
            self.lock.release()
        
    def removeProgressBar(self, pbarobj):
        ## debug.fmsg("removeProgressBar")
        self.lock.acquire()
        try:
            if not pbarobj.defunct:
            ## called by progressbar idle function, in the main thread
                self.listofprogressobjects.remove(pbarobj) ## removes progress bar from list
                pbarobj.defunct = 1
        finally:
            self.lock.release()
        

    def stopAll(self, widget = None):
        ## called by activityViewer and by quit.
        ## run by main thread only
        self.lock.acquire()
        ## debug.fmsg("stopAll")
        try:
            for thisobj in self.listofprogressobjects:
                if thisobj.get_success() == 0 :
                    thisobj.stop_it() ## stops everything (no dismiss)
        finally:
            self.lock.release()
    def cancelAll(self):
        ## debug.fmsg("cancelAll")
        ## for internal use only.
        ## Must run in main thread
        self.lock.acquire()
        try:
            for worker in self.listofworkers :
                worker.cancel()
        finally:
            self.lock.release()
            
    def newProgressBar(self, newpobj):
        ## called by every worker
        ## debug.fmsg("newProgressBar")
        self.lock.acquire()
        try:
            self.listofprogressobjects.append(newpobj)
        finally:
            self.lock.release()
        switchboard.notify("new progressbar", newpobj)
        
        
        
    def NumThreadsInProgress(self):
        ## Calculates how many threads are running.
        ## only used by activityViewer
        numthreadinprogress = 0
        ## debug.fmsg(" NumThreadsInProgress")
        self.lock.acquire()
        try:
            for thisthread in self.listofprogressobjects:
                if thisthread.get_success() == 0:
                    numthreadinprogress +=1
        finally:
            self.lock.release()
        return numthreadinprogress

    def NumThreadsDone(self):
        ## Calculates how many threads exist in the list that are finished.
        numthreadsdone = 0
        ## debug.fmsg("NumThreadsDone")
        self.lock.acquire()
        try:
            for thisthread in self.listofprogressobjects:
                if thisthread.get_success() != 0:
                    numthreadsdone +=1
        finally:
            self.lock.release()
            
        return numthreadsdone

    def newWorker (self, newworker):
        ## print "adding worker"
        ## debug.fmsg("newWorker")
        self.lock.acquire()
        try:
            self.listofworkers.append(newworker)
        finally:
            self.lock.release()

    def removeWorker(self, oldworker):
        ## print "removing worker"
        ## debug.fmsg("removeWorker")
        self.lock.acquire()
        try:
            try:
                self.listofworkers.remove(oldworker)
            except Exception:
                pass
        finally:
            self.lock.release()


threadManager = ThreadManager() 

##import atexit
##atexit.register(threadManager.quit) ## automatically stop any running thread that was left behind at exit.

