# -*- python -*-
# $RCSfile: equationPage.py,v $
# $Revision: 1.48 $
# $Author: langer $
# $Date: 2006/08/29 18:43:25 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import devel
from oof2.SWIG.common import switchboard
from oof2.SWIG.engine import equation
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common import utils
from oof2.common.IO import mainmenu
from oof2.common.IO import whoville
from oof2.common.IO.GUI import gtklogger
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import whowidget
from oof2.common.IO.GUI import parameterwidgets
from oof2.engine import skeletoncontext
from oof2.engine import subproblemcontext
from oof2.engine.IO import subproblemmenu
import gtk
import oof2.engine.mesh

meshmenu = mainmenu.OOF.Mesh

class ButtonSignal:
    def __init__(self, button, signal):
        self.button = button
        self.signal = signal
    def set(self, active):
        debug.mainthreadTest()
        self.signal.block()
        self.button.set_active(active)
        self.signal.unblock()
    def set_sensitive(self, sensitivity):
        debug.mainthreadTest()
        self.button.set_sensitive(sensitivity)

class EquationPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Equations", ordering=220,
                                 tip="Define equations for the mesh to solve.")
        mainbox = gtk.VBox()
        self.gtk.add(mainbox)
        
        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        if devel.devel()>=2:
            self.meshwidget = whowidget.WhoWidget(
                whoville.getClass('SubProblemWho'), callback=self.meshCB,
                scope=self)
        else:
            self.meshwidget = whowidget.WhoWidget(oof2.engine.mesh.meshes,
                                              callback=self.meshCB, scope=self)
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[0], expand=0, fill=0)
        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[1], expand=0, fill=0)
        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[2], expand=0, fill=0)
        if devel.devel()>=2:
            label = gtk.Label("SubProblem=")
            label.set_alignment(1.0, 0.5)
            centerbox.pack_start(label, expand=0, fill=0)
            centerbox.pack_start(self.meshwidget.gtk[3], expand=0, fill=0)

        eqnframe = gtk.Frame("Equations")
        eqnframe.set_shadow_type(gtk.SHADOW_IN)
        scroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(scroll, "Equations")
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        eqnframe.add(scroll)
        bbox = gtk.VBox() # extra layer keeps table from expanding inside scroll
        scroll.add_with_viewport(bbox)
        self.eqntable = gtk.Table()
        bbox.pack_start(self.eqntable, expand=0, fill=0)
        self.eqnbuttons = {}
        self.build_eqnTable()
        switchboard.requestCallbackMain("new equation",
                                        self.newEquationCB)
        switchboard.requestCallbackMain("equation activated",
                                        self.activateEqnCB)
        switchboard.requestCallbackMain("kinetics activated",
                                        self.activateKinCB)
        switchboard.requestCallbackMain("dynamics activated",
                                        self.activateDynCB)
        mainbox.pack_start(eqnframe, expand=1, fill=1)

        copybox = gtk.HBox(spacing=3)
        self.copybutton = gtk.Button("Copy Equation State")
        gtklogger.setWidgetName(self.copybutton, "Copy")
        gtklogger.connect(self.copybutton, "clicked", self.copyeqstateCB)
        copybox.pack_start(self.copybutton, expand=1,fill=0)
        mainbox.pack_start(copybox, expand=0, fill=0)
        guitop.top().tooltips.set_tip(self.copybutton,
                                      "Copy the status of all equations from the current mesh to another mesh.")
        
        self.sensitize()

        switchboard.requestCallbackMain(("new who", "Skeleton"),
                                        self.newSkeleton)
        switchboard.requestCallbackMain(("new who", "Mesh"),
                                        self.newMesh)
        switchboard.requestCallbackMain(("remove who", "Mesh"),
                                        self.removeMesh)
        switchboard.requestCallbackMain("made reservation",
                                        self.reservationChanged)
        switchboard.requestCallbackMain("cancelled reservation",
                                        self.reservationChanged)

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    #######################

    def build_eqnTable(self):
        debug.mainthreadTest()
        self.eqntable.foreach(gtk.Object.destroy) # clear the table
        self.eqnbuttons = {}
        eqlist = equation.allEquations
        self.eqntable.resize(len(eqlist), 3)
        row=0
        for eqn in eqlist:
            label = gtk.Label(utils.underscore2space(eqn.name()))
            label.set_alignment(1.0, 0.5)
            self.eqntable.attach(label, 0,1, row, row+1,
                                 xoptions=gtk.FILL, yoptions=0)
            button = gtk.CheckButton('active')
            gtklogger.setWidgetName(button, eqn.name() + " active")
            signal = gtklogger.connect(button, 'clicked', self.eqnButtonCB, eqn)
            self.eqnbuttons[(eqn.name(), "active")] = ButtonSignal(button,
                                                                   signal)
            guitop.top().tooltips.set_tip(button,
                                          'Active equations will be solved.')
            self.eqntable.attach(button, 2,3, row,row+1, xoptions=0,
                                 yoptions=0)
            if eqn.get_kinetic()>0 and devel.devel()>=1:
                button = gtk.CheckButton('kinetic')
                gtklogger.setWidgetName(button, eqn.name() + " kinetic")
                signal = gtklogger.connect(button, 'clicked', self.eqnKineticCB,
                                           eqn)
                self.eqnbuttons[(eqn.name(), "kinetic")] = ButtonSignal(button,
                                                                        signal)
                self.eqntable.attach(button, 3,4, row, row+1,
                                   xoptions=0, yoptions=0)
            if eqn.get_dynamics()>0 and devel.devel()>=1:
                button = gtk.CheckButton('dynamic')
                gtklogger.setWidgetName(button, eqn.name() + " dynamic")
                signal = gtklogger.connect(button, 'clicked',
                                           self.eqnDynamicCB, eqn)
                self.eqnbuttons[(eqn.name(), "dynamic")] = ButtonSignal(button,
                                                                        signal)
                self.eqntable.attach(button, 4,5, row, row+1,
                                   xoptions=0, yoptions=0)
            row += 1
        self.eqntable.attach(gtk.VSeparator(), 1,2, 0,len(eqlist),
                             xoptions=0, yoptions=gtk.EXPAND|gtk.FILL)
        self.eqntable.set_col_spacing(0, 3)

    def newEquationCB(self):  # Switchboard, "new equation".
        self.build_eqnTable()
        self.show()


    def eqnKineticCB(self, button, eqn): # gtk callback
        if button.get_active():
            if devel.devel()>=2:
                subproblemmenu.eqnmenu.ActivateKinetics(
                    subproblem = self.currentFullSubProblemName(),
                    equation=eqn)
            else:
                meshmenu.Equation.ActivateKinetics(
                    mesh=self.currentFullMeshName(), equation=eqn)
        else:
            if devel.devel()>=2:
                subproblemmenu.eqnmenu.DeactivateKinetics(
                    subproblem = self.currentFullSubProblemName(),
                    equation=eqn)
            else:
                meshmenu.Equation.DeactivateKinetics(
                    mesh=self.currentFullMeshName(), equation=eqn)


    def eqnDynamicCB(self, button, eqn): # gtk callback
        if button.get_active():
            if devel.devel()>=2:
                subproblemmenu.eqnmenu.ActivateDynamics(
                    subproblem = self.currentFullSubProblemName(),
                    equation=eqn)
            else:
                meshmenu.Equation.ActivateDynamics(
                    mesh=self.currentFullMeshName(), equation=eqn)
        else:
            if devel.devel()>=2:
                subproblemmenu.eqnmenu.DeactivateDynamics(
                    subproblem = self.currentFullSubProblemName(),
                    equation=eqn)
            else:
                meshmenu.Equation.DeactivateDynamics(
                    mesh=self.currentFullMeshName(), equation=eqn)


    def eqnButtonCB(self, button, eqn): # gtk callback
        if button.get_active():
            if devel.devel()>=2:
                subproblemmenu.eqnmenu.Activate(
                    subproblem = self.currentFullSubProblemName(),
                    equation=eqn)
            else:
                meshmenu.Equation.Activate(
                    mesh=self.currentFullMeshName(), equation=eqn)
        else:
            if devel.devel()>=2:
                subproblemmenu.eqnmenu.Deactivate(
                    subproblem = self.currentFullSubProblemName(),
                    equation=eqn)
            else:
                meshmenu.Equation.Deactivate(
                    mesh=self.currentFullMeshName(), equation=eqn)


    def activateEqnCB(self, mesh, eqn, active): # sb "equation activated"
        if mesh == self.currentFullMeshName():
            self.eqnbuttons[(eqn, "active")].set(active)
            
    def activateKinCB(self, mesh, eqn, active): # sb "kinetics activated"
        if mesh == self.currentFullMeshName():
            self.eqnbuttons[(eqn, "kinetic")].set(active)
    def activateDynCB(self, mesh, eqn, active): # sb "dynamics activated"
        if mesh == self.currentFullMeshName():
            self.eqnbuttons[(eqn, "dynamic")].set(active)
    ########################

    def currentMSName(self):
        path = labeltree.makePath(self.meshwidget.get_value(depth=1))
        if path:
            return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonName(self):
        path = labeltree.makePath(self.meshwidget.get_value(depth=2))
        if path:
            return path[1]
    def currentSkeleton(self):
        ms = self.currentMS()
        meshname = self.currentSkeletonName()
        if meshname:
            return skeletoncontext.getSkeleton(ms, meshname)

    def currentFullMeshName(self):
        if devel.devel()>=2:
            return self.meshwidget.get_value(depth=3)
##            pname = self.meshwidget.get_value()
##            path = labeltree.makePath(pname)
##            if path:
##                return path[0] + ':' + path[1] + ':' + path[2]
        else:
            return self.meshwidget.get_value()
    def currentFullSubProblemName(self):
        return self.meshwidget.get_value()
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        if path:
            return path[2]
    def currentMesh(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()].getObject()
        except KeyError:
            return None
    def currentMeshContext(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
            
    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        meshok = skelok and self.currentMesh() is not None and \
                 not self.currentMeshContext().query_reservation()
        self.eqntable.set_sensitive(meshok)
        self.copybutton.set_sensitive(meshok)
        self.update()
    def update(self):
        self.set_state(self.currentFullMeshName())
    def currentFullSubProblemName(self):
        return self.meshwidget.get_value()
    def set_state(self, meshpath):
        debug.mainthreadTest()
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        if path and path[-1]:                    # ie, mesh name is not empty
            themesh = oof2.engine.mesh.meshes[path]
            skel = themesh.skeleton
            for eqn in equation.allEquations:
                if devel.devel()>=2:
                    try:
                        spblmwho = subproblemcontext.subproblems[labeltree.makePath(self.currentFullSubProblemName())]
                    except KeyError:
                        return
                    active = eqn.name() in spblmwho.getObject().active_equations
                else:
                    active = themesh.getObject().is_active_equation(eqn)
                self.eqnbuttons[(eqn.name(),"active")].set(active)
                if devel.devel()>=2 and eqn.get_kinetic():
                    kinstate = eqn.name() in spblmwho.getObject().active_kinetics
                    if active and eqn.get_kinetic():
                        self.eqnbuttons[(eqn.name(),"kinetic")].set_sensitive(1)
                    else:
                        self.eqnbuttons[(eqn.name(),"kinetic")].set_sensitive(0)
                        
                    ## kactive = themesh.getObject().is_kinetically_active_equation(eqn)
                   ## print "active kinetics", eqn.name(), active
                    self.eqnbuttons[(eqn.name(),"kinetic")].set(kinstate)
                   
                if devel.devel()>=2 and eqn.get_dynamics():
                    dynstate = eqn.name() in spblmwho.getObject().active_dynamics
                    if active and eqn.get_dynamics():
                        self.eqnbuttons[(eqn.name(),"dynamic")].set_sensitive(1)
                    else:
                        self.eqnbuttons[(eqn.name(),"dynamic")].set_sensitive(0)
                    ## dyactive = themesh.getObject().is_dynamically_active_equation(eqn)
                   ## print "active dynamics", eqn.name(), active
                    
                    self.eqnbuttons[(eqn.name(),"dynamic")].set(dynstate)
                   
                   ## print "after setting dyn equation active"
        else:                           # no current mesh
            for button in self.eqnbuttons.values():
                button.set(0)

    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        if not self.currentMesh():
            self.meshwidget.set_value(skeletonname)

    def copyeqstateCB(self, gtkobj):
        if devel.devel()>=2:
            menuitem = subproblemmenu.eqnmenu.Copy_Equation_State
            targetparam = menuitem.get_arg("target")
            targetparam.set(self.currentFullSubProblemName())
            if parameterwidgets.getParameters(targetparam,
                                              title="Select a target subproblem"):
                menuitem.callWithDefaults(source=self.currentFullSubProblemName())
        else:
            menuitem = meshmenu.Copy_Equation_State
            targetparam = menuitem.get_arg("target")
            targetparam.set(self.currentFullMeshName())
            if parameterwidgets.getParameters(targetparam,
                                              title="Select a target mesh"):
                menuitem.callWithDefaults(source=self.currentFullMeshName())
    
    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()
        
    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def removeMesh(self, path):    # switchboard ("remove who", "Mesh")
        self.update()
        self.sensitize()

    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullMeshName() == whoobj.path():
            self.sensitize()


#############

ep = EquationPage()
