# -*- python -*-
# $RCSfile: fieldPage.py,v $
# $Revision: 1.46 $
# $Author: langer $
# $Date: 2006/08/29 18:43:25 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.SWIG.common import devel
from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common.IO import mainmenu
from oof2.common.IO import parameter
from oof2.common.IO import whoville
from oof2.common.IO.GUI import gtklogger
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import whowidget
from oof2.engine import fieldinit
from oof2.engine import skeletoncontext
from oof2.engine.IO import subproblemmenu
import gobject
import gtk
import oof2.engine.mesh
import oof2.SWIG.engine.field
import string

allFields = oof2.SWIG.engine.field.allFields

meshmenu = mainmenu.OOF.Mesh

class ButtonSignal:
    def __init__(self, button, signal):
        self.button = button
        self.signal = signal
    def set(self, active):
        debug.mainthreadTest()
        self.signal.block()
        self.button.set_active(active)
        self.signal.unblock()
    def set_sensitive(self, sensitivity):
        debug.mainthreadTest()
        self.button.set_sensitive(sensitivity)


class FieldPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Fields", ordering=210,
                                 tip="Define fields on a finite element mesh.")
        mainbox = gtk.VBox(spacing=2)
        self.gtk.add(mainbox)

        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        if devel.devel()>=2:
            self.meshwidget = whowidget.WhoWidget(
                whoville.getClass('SubProblemWho'),
                callback=self.meshCB, scope=self)
        else:
            self.meshwidget = whowidget.WhoWidget(oof2.engine.mesh.meshes,
                                              callback=self.meshCB, scope=self)
        switchboard.requestCallbackMain(self.meshwidget, self.meshWidgetCB)
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[0], expand=0, fill=0)
        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[1], expand=0, fill=0)
        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[2], expand=0, fill=0)
        if devel.devel()>=2:
            label = gtk.Label("SubProblem=")
            label.set_alignment(1.0, 0.5)
            centerbox.pack_start(label, expand=0, fill=0)
            centerbox.pack_start(self.meshwidget.gtk[3], expand=0, fill=0)

        vpane = gtk.VPaned()
        gtklogger.setWidgetName(vpane, 'Pane')
        mainbox.pack_start(vpane, expand=1, fill=1, padding=3)
##        vpane.set_gutter_size(10)

        fieldframe = gtk.Frame("Fields")
        fieldframe.set_shadow_type(gtk.SHADOW_IN)
        vpane.pack1(fieldframe, resize=1, shrink=0)
        gtklogger.connect_passive(fieldframe, 'size-allocate')
        scroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(scroll, "Fields")
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        fieldframe.add(scroll)
        bbox = gtk.VBox()
        scroll.add_with_viewport(bbox)
        self.fieldtable = gtk.Table()
        bbox.pack_start(self.fieldtable, expand=0, fill=0)
        self.build_fieldTable()

        frame = gtk.Frame("Field Initialization")
        frame.set_shadow_type(gtk.SHADOW_IN)
        vpane.pack2(frame, resize=1, shrink=0)
        ivbox = gtk.VBox()
        frame.add(ivbox)
        self.initscroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(self.initscroll, "InitScroll")
        self.initscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.initscroll.set_shadow_type(gtk.SHADOW_IN)
        ivbox.pack_start(self.initscroll, expand=1, fill=1)

        # The ListStore just contains the defined Fields.  The
        # TreeView displays their names and initializers.
        self.initlist = gtk.ListStore(gobject.TYPE_PYOBJECT)
        self.initview = gtk.TreeView(self.initlist)
        gtklogger.setWidgetName(self.initview, 'Initializers')
        self.initscroll.add(self.initview)
        self.initview.set_headers_clickable(False)
        fieldnamecell = gtk.CellRendererText()
        fieldnamecol = gtk.TreeViewColumn('Field')
        self.initview.append_column(fieldnamecol)
        fieldnamecol.pack_start(fieldnamecell, expand=False)
        fieldnamecol.set_cell_data_func(fieldnamecell, self.renderFieldName)
                                        
        fieldinitcell = gtk.CellRendererText()
        fieldinitcol = gtk.TreeViewColumn('Initializer')
        self.initview.append_column(fieldinitcol)
        fieldinitcol.pack_start(fieldinitcell, expand=True)
        fieldinitcol.set_cell_data_func(fieldinitcell, self.renderFieldInit)

        selection = self.initview.get_selection()
        gtklogger.adoptGObject(selection, self.initview,
                               access_method=self.initview.get_selection)
        self.initselsignal = gtklogger.connect(selection, 'changed',
                                               self.initSelectCB)
        gtklogger.connect(self.initview, 'row-activated',
                          self.initActivateRowCB)
        
        self.initmsg = gtk.Label("Fields defined above\ncan be initialized here\nafter a Mesh is created.")
        ivbox.pack_start(self.initmsg, expand=1, fill=1)

        bbox = gtk.HBox()
        ivbox.pack_start(bbox, expand=0, fill=0, padding=2)

        self.initbutton = gtk.Button('Set Initializer...')
        gtklogger.setWidgetName(self.initbutton, 'Set Init')
        gtklogger.connect(self.initbutton, 'clicked', self.initbuttonCB)
        guitop.top().tooltips.set_tip(self.initbutton,
                                      'Click to initialize the selected field. Alternatively, a double-click on the desired field brings up initializer.')
        bbox.pack_start(self.initbutton, expand=0, fill=0, padding=2)

        self.copybutton = gtk.Button("Copy Field State...")
        gtklogger.setWidgetName(self.copybutton, 'CopyField')
        gtklogger.connect(self.copybutton, 'clicked', self.copyfstateCB)
        bbox.pack_end(self.copybutton, expand=0, fill=0, padding=2)
        guitop.top().tooltips.set_tip(self.copybutton,
                                      "Copy all field status variables from the current mesh to another mesh.")

        self.sensitize()

        switchboard.requestCallbackMain(("new who", "Skeleton"),
                                        self.newSkeleton)
        switchboard.requestCallbackMain(("new who", "Mesh"),
                                        self.newMesh)
##        switchboard.requestCallbackMain(("remove who", "Mesh"),
##                                        self.removeMesh)
        switchboard.requestCallbackMain("made reservation",
                                        self.reservationChanged)
        switchboard.requestCallbackMain("cancelled reservation",
                                        self.reservationChanged)
        switchboard.requestCallbackMain("new field",
                                        self.newFieldCB)
        switchboard.requestCallbackMain("field defined",
                                        self.defineFldCB)
        switchboard.requestCallbackMain("field activated",
                                        self.activateFldCB)
        switchboard.requestCallbackMain("field inplane",
                                        self.inplaneFldCB)
        switchboard.requestCallbackMain("field initialized",
                                        self.initFldCB)

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    def build_fieldTable(self):
        debug.mainthreadTest()
        self.fieldtable.foreach(gtk.Object.destroy) # clear the table
        self.fieldbuttons = {}
        self.fieldtable.resize(len(allFields), 6)
        self.fieldtable.attach(gtk.VSeparator(), 1,2, 0,len(allFields),
                               xoptions=0, yoptions=gtk.EXPAND|gtk.FILL)
        self.fieldtable.set_col_spacing(0, 3)
        row = 0
        for fld in allFields:
            label = gtk.Label(fld.name())
            label.set_alignment(1.0, 0.5)
            self.fieldtable.attach(label, 0,1, row,row+1,
                                   xoptions=gtk.FILL, yoptions=0)
            button = gtk.CheckButton('defined')
            gtklogger.setWidgetName(button, fld.name()+" defined")
            signal = gtklogger.connect(button, 'clicked',
                                       self.fieldDefineCB, fld)
            self.fieldbuttons[(fld.name(), "defined")] = ButtonSignal(button,
                                                                      signal)
            self.fieldtable.attach(button, 2,3, row, row+1,
                                   xoptions=0, yoptions=0)
            self.setFieldDefineTip(button, fld)

            button = gtk.CheckButton('active')
            gtklogger.setWidgetName(button, fld.name()+" active")
            signal = gtklogger.connect(button, 'clicked',
                                       self.fieldActiveCB, fld)
            self.fieldbuttons[(fld.name(), "active")] = ButtonSignal(button,
                                                                     signal)
            self.fieldtable.attach(button, 3,4, row, row+1,
                                   xoptions=0, yoptions=0)
            self.setFieldActiveTip(button, fld)

            button = gtk.CheckButton('in-plane')
            gtklogger.setWidgetName(button, fld.name() + " in-plane")
            signal = gtklogger.connect(button, 'clicked',
                                       self.fieldInPlaneCB, fld)
            self.fieldbuttons[(fld.name(), "inplane")] = ButtonSignal(button,
                                                                      signal)
            self.fieldtable.attach(button, 4,5, row, row+1,
                                   xoptions=0, yoptions=0)
            self.setFieldInPlaneTip(button, fld)

            row += 1

    def newFieldCB(self):               # switchboard "new field"
        self.build_fieldTable()
        self.show()

    def fieldDefineCB(self, button, field): # gtk callback
        if button.get_active():
            meshmenu.Field.Define(mesh=self.currentFullMeshName(),
                                  field=field)
        else:
            meshmenu.Field.Undefine(mesh=self.currentFullMeshName(),
                                    field=field)
        self.setFieldDefineTip(button, field)

    def setFieldDefineTip(self, button, field):
        if button.get_active():
            verb = "Undefine"
        else:
            verb = "Define"
        guitop.top().tooltip(button, "%s the %s field on the mesh.  Only defined fields have values." % (verb, field.name()))

    def fieldActiveCB(self, button, field): # gtk callback
        if button.get_active():
            if devel.devel()>=2:
                subproblemmenu.fieldmenu.Activate(
                    subproblem=self.currentFullSubProblemName(), field=field)
            else:
                meshmenu.Field.Activate(
                    mesh=self.currentFullMeshName(), field=field)
        else:
            if devel.devel()>=2:
                subproblemmenu.fieldmenu.Deactivate(
                    subproblem=self.currentFullSubProblemName(), field=field)
            else:
                meshmenu.Field.Deactivate(
                    mesh=self.currentFullMeshName(), field=field)
        self.setFieldActiveTip(button, field)
        
    def setFieldActiveTip(self, button, field):
        if button.get_active():
            verb = "Deactivate"
        else:
            verb = "Activate"
        guitop.top().tooltip(button, "%s the %s field on the mesh. The solver finds the values of the active fields by solving the active equations." % (verb, field.name()))

    def fieldInPlaneCB(self, button, field): # gtk callback
        if button.get_active():
            if devel.devel()>=2:
                subproblemmenu.fieldmenu.In_Plane(subproblem=self.currentFullSubProblemName(),
                                                  field = field)
            else:
                meshmenu.Field.In_Plane(mesh=self.currentFullMeshName(),
                                        field=field)
        else:
            if devel.devel()>=2:
                subproblemmenu.fieldmenu.Out_of_Plane(subproblem=self.currentFullSubProblemName(),
                                                      field = field)
            else:
                meshmenu.Field.Out_of_Plane(mesh=self.currentFullMeshName(),
                                            field=field)
        self.setFieldInPlaneTip(button, field)
        
    def setFieldInPlaneTip(self, button, field):
        if button.get_active():
            verb = "Do not constrain"
        else:
            verb = "Constrain"
        guitop.top().tooltip(button, "%s the %s field to lie in the x-y plane."
                             % (verb, field.name()))

    def defineFldCB(self, meshname, fieldname, defined): # sb "field defined"
        if meshname == self.currentFullMeshName():
            self.initselsignal.block()
            try:
                self.fieldbuttons[(fieldname, "defined")].set(defined)
                self.update()
            finally:
                self.initselsignal.unblock()

    def activateFldCB(self, meshname, fieldname, active): # sb "field activated"
        if devel.devel()>=2:
            if meshname == self.currentFullSubProblemName():
                self.fieldbuttons[(fieldname, "active")].set(active)
        else:
            if meshname == self.currentFullMeshName():
                self.fieldbuttons[(fieldname, "active")].set(active)

    def inplaneFldCB(self, meshname, fieldname, inplane): # sb "field inplane"
        if devel.devel()>=2:
            if meshname == self.currentFullSubProblemName():
                self.fieldbuttons[(fieldname, "inplane")].set(inplane)
        else:
            if meshname == self.currentFullMeshName():
                self.fieldbuttons[(fieldname, "inplane")].set(inplane)

    ########################

    def currentMSName(self):
        path = labeltree.makePath(self.meshwidget.get_value(depth=1))
        if path:
            return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonName(self):
        path = labeltree.makePath(self.meshwidget.get_value(depth=2))
        if path:
            return path[1]
    def currentSkeleton(self):
        ms = self.currentMS()
        skelname = self.currentSkeletonName()
        if skelname:
            return skeletoncontext.getSkeleton(ms, skelname)

    def currentFullMeshName(self):
        if devel.devel()>=2:
            return self.meshwidget.get_value(depth=3)
        else:
            return self.meshwidget.get_value()
        
    def currentFullSubProblemName(self):
        return self.meshwidget.get_value()
    def currentMeshName(self):
        path = labeltree.makePath(self.meshwidget.get_value(depth=3))
        if path:
            return path[2]
    def currentMeshContext(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
    def currentMesh(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()].getObject()
        except KeyError:
            return None
            
    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        meshok = skelok and self.currentMesh() is not None and \
                 not self.currentMeshContext().query_reservation()

        self.fieldtable.set_sensitive(meshok)
        self.initview.set_sensitive(meshok)
        self.initbutton.set_sensitive(meshok and
                                      self.selectedField() is not None)

        self.copybutton.set_sensitive(meshok)
        
    def update(self):
        self.set_state(self.currentFullMeshName())

    def set_state(self, meshpath):
        debug.mainthreadTest()
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        if path and path[-1]:           # ie, mesh name is not empty
            if devel.devel()>=2:
                try:
                    spcontext = oof2.engine.subproblemcontext.subproblems[path]
                    themesh = spcontext.getMesh()
                except KeyError:
                    themesh = oof2.engine.mesh.meshes[path]
            else:
                themesh = oof2.engine.mesh.meshes[path] # Who object
            femesh = themesh.getObject() # FEMesh object
            skel = themesh.skeleton
            for fld in allFields:
                fname = fld.name()
                fdef = femesh.is_defined_field(fld)
                self.fieldbuttons[(fname, "defined")].set(fdef)
                self.fieldbuttons[(fname, "active")].set_sensitive(fdef)
                self.fieldbuttons[(fname, "inplane")].set_sensitive(fdef)
                if fdef:
                    if devel.devel()>=2:
                        try:
                            sp = spcontext.getObject()
                            self.fieldbuttons[(fname, "active")].set(
                                sp.is_active_field(fld))
                            self.fieldbuttons[(fname, "inplane")].set(
                                sp.is_in_plane(fld))
                        except UnboundLocalError:
                            pass
                    else:
                        self.fieldbuttons[(fname, "active")].set(
                            femesh.is_active_field(fld))
                        self.fieldbuttons[(fname, "inplane")].set(
                            femesh.in_plane(fld))
                else:                   # field not defined
                    self.fieldbuttons[(fname, "active")].set(0)
                    self.fieldbuttons[(fname, "inplane")].set(0)
            self.updateInitializers()
            self.initmsg.hide()
            self.initscroll.show_all()

        else:                           # no current mesh
            self.initscroll.hide()
            self.initmsg.show()
            for button in self.fieldbuttons.values():
                button.set(0)

    # Functions that the TreeView machinery calls to fill in the
    # cells.  These will never be called for undefined fields, so
    # there's no need to check that the mesh or field exists.
    def renderFieldName(self, column, cell_renderer, model, iter):
        fld = model[iter][0]
        cell_renderer.set_property('text', fld.name())
    def renderFieldInit(self, column, cell_renderer, model, iter):
        fld = model[iter][0]
        mesh = self.currentMeshContext()
        femesh = mesh.getObject()
        init = mesh.get_initializer(fld)
        if init:
            cell_renderer.set_property('text', init.shortrepr())
        else:
            cell_renderer.set_property('text', '<uninitialized>')
    def updateInitializers(self):
        currentField = self.selectedField()
        self.initlist.clear()
        mesh = self.currentMeshContext()
        if mesh:
            femesh = mesh.getObject()
            for fld in allFields:
                if femesh.is_defined_field(fld):
                    self.initlist.append([fld])
            if currentField is not None:
                for row in range(len(self.initlist)):
                    if self.initlist[row][0] == currentField:
                        self.initview.get_selection().select_path(row)

    def copyfstateCB(self, gtkobj): # Button callback.
        if devel.devel()>=2:
            menuitem = subproblemmenu.fieldmenu.Copy_Field_State
            targetparam = menuitem.get_arg("target")
            
            targetparam.set(self.currentFullSubProblemName())
            if parameterwidgets.getParameters(targetparam,
                                              title="Select a target mesh"):
                menuitem.callWithDefaults(source=self.currentFullSubProblemName())
        else:
            menuitem = meshmenu.Copy_Field_State
            targetparam = menuitem.get_arg("target")
            
            targetparam.set(self.currentFullMeshName())
            if parameterwidgets.getParameters(targetparam,
                                              title="Select a target mesh"):
                menuitem.callWithDefaults(source=self.currentFullMeshName())
                

    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        if not self.currentMeshContext():
            self.meshwidget.set_value(skeletonname)

    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()
        
    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def meshWidgetCB(self, interactive):      # switchboard callback
        self.update()
        self.sensitize()

##    def removeMesh(self, path):    # switchboard ("remove who", "Mesh")
##        self.update()
##        self.sensitize()

    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullMeshName() == whoobj.path():
            self.sensitize()

    def initSelectCB(self, selection):
        self.sensitize()

    def initActivateRowCB(self, treeview, path, col):
        self.initbuttonCB()
            
    # Get the Field currently selected in the initializer TreeView
    def selectedField(self):
        selection = self.initview.get_selection() # TreeSelection object
        model, iter = selection.get_selected()
        if iter:
            return model[iter][0]

    def initbuttonCB(self, *args):      # gtk button callback
        menuitem = meshmenu.Initialize
        fld = self.selectedField()
        # We can't simply use
        # parameter.getParameters(menuitem.get_arg('initializer'))
        # here, because the menu items parameter accepts members of
        # the FieldInit base class, but we need to create a widget for
        # the appropriate derived class (ScalarFieldInit, et al).
        # It's the derived classes that are the registered classes.
        param = parameter.RegisteredParameter(
            'initializer', fieldinit.fieldInitDict[fld.classname()])

        # Set the parameter value to the current initializer of the
        # appropriate field in the current mesh (context) -- this is
        # the value that the initializer widget will start with.  If
        # it doesn't get set here, it will just have the most
        # recently-set one, which may be for a different mesh.
        param.value = self.currentMeshContext().get_initializer(fld)

        init = parameterwidgets.getParameterValues(
            param,
            title = 'Initialize field ' + self.selectedField().name(),
            scope=self)
        
        if init is not None:
            menuitem(mesh=self.currentFullMeshName(),
                     field=fld,
                     initializer=init[0])

    def initFldCB(self):                # sb "field initialized"
        # This function really should just emit the ListStore signal
        # that causes its TreeView to update.  Presumably that's easy
        # to do, but I don't know how at the moment, so I'll just
        # rebuild the list entirely.
        self.updateInitializers()


#############

fp = FieldPage()
