# -*- python -*-
# $RCSfile: meshIPC.py,v $
# $Revision: 1.9 $
# $Author: rlua $
# $Date: 2006/07/03 22:04:07 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

#
# Allocate nodes and submesh for a process
#

from oof2.common.IO import oofmenu
from oof2.SWIG.common import mpitools
from oof2.common.IO import parallelmainmenu
from oof2.engine import skeletonelement
from oof2.common.IO import automatic
from oof2.common.IO import parameter
from oof2.SWIG.engine import masterelement
from oof2.common import labeltree
from oof2.engine import skeletoncontext
from oof2.engine.IO import meshparameters
from oof2.common.IO import whoville
from oof2.SWIG.common import switchboard
from oof2.common.IO import reporter
import oof2.engine.mesh
import string

StringParameter = parameter.StringParameter

## General definitions
_rank = mpitools.Rank()
_size = mpitools.Size()

## OOF.LoadData.IPC.Mesh
ipcmeshmenu = parallelmainmenu.ipcmenu.addItem(
    oofmenu.OOFMenuItem('Mesh', secret=1, no_log=1)
    )

#
# This is the callback for OOF.LoadData.IPC.Mesh.femesh
# This is run in every processor.
#
def parallel_femesh(menuitem,name,skeleton,T_typename,Q_typename):
    # Build edict map. Access masterelement dictionary directly
    MEdict=masterelement.getMasterElementDict()
    edict = {
        MEdict[T_typename].nsides(): MEdict[T_typename],
        MEdict[Q_typename].nsides(): MEdict[Q_typename]
        }
    skelpath = labeltree.makePath(skeleton)
    skel = skeletoncontext.skeletonContexts[skelpath].getObject()
    femesh = skel.femesh_shares(edict, skeletonelement.SkeletonElement.realmaterial)
    if _rank == 0:
        femesh.all_meshskeletons = skel.all_skeletons
    #collect_pieces(femesh)
    if femesh is not None:
        oof2.engine.mesh.meshes.add(
            skelpath+[name], femesh,
            parent=skeletoncontext.skeletonContexts[skelpath],
            skeleton=skel,
            elementdict=edict,
            materialfactory=skeletonelement.SkeletonElement.realmaterial)

#
# This menu command called by parallel_newMesh in this file
#
## OOF.LoadData.IPC.Mesh.femesh
# secret and no_log inherited from ipcmeshmenu (above)?
ipcmeshmenu.addItem(oofmenu.OOFMenuItem(
    'femesh',
    callback = parallel_femesh,
    threadable = oofmenu.PARALLEL_THREADABLE,
    params =
    [
    StringParameter('name'),
    StringParameter('skeleton'),
    StringParameter('T_typename'),
    StringParameter('Q_typename')
    ]
    ))

#
# This function called in meshmenu.py
#
def parallel_newMesh(name, skeleton, elem_typename1, elem_typename2):
    ipcmeshmenu.femesh(name=name,
                       skeleton=skeleton,
                       T_typename=elem_typename1,
                       Q_typename=elem_typename2)


# This method based on the one with the same name from skeletonIPC.py
# A function to collect partitioned mesh skeleton -- not to be used in the end
def collect_pieces(femesh):

    # Build dictionary (instead of using the mesh assigned indices (via index() or get_index())
    #  which are unique but may have gaps)
    nodedict = {}
    i = 0
    for node in femesh.node_iterator():
        nodedict[node.index()] = i
        i += 1

    global _rank
    global _size
    # Gather minimal info for element polygons (to display mesh Skeleton at #0)
    # Only the #0 will store the reconstituted information.

    #RCL: Collect number of nodes from each process into an array.
    # One gets this for nnodes: [nnodes0, nnodes1, ...]
    nnodes = mpitools.Allgather_Int(femesh.nnodes())
    
    #RCL: Same for the elements.
    # One gets this for nelems: [nelems0, nelems1, ...]
    nelems = mpitools.Allgather_Int(femesh.nelements())
    myCoords = reduce(
        lambda x,y: x+y, [[nd.position().x,
                           nd.position().y]
                          for nd in femesh.node_iterator()]
        )
    
    coordSizes = [i*2 for i in nnodes]
    #RCL: Collect (x,y) coordinates of nodes from each process
    # coordSizes contains the array of number of coordinates (counting x and y separately) from each process
    # The return value is a 2-D array, indexed first by the process number(?)
    # One gets this for allCoords: [[x0,y0,x1,y1,...], [x0',y0',x1',y1',...], ...]
    allCoords = mpitools.Allgather_DoubleVec(myCoords,
                                             size_known=coordSizes)

    #RCL: One gets the following format after the list comprehension operations
    # allCoords = [[(x0,y0),(x1,y1), ...], [...], ...]
    allCoords = [ [(allCoords[i][2*j],allCoords[i][2*j+1])
                   for j in range(nnodes[i])]
                  for i in range(_size) ]
    
    # element connectivity signature
    myEConSigs = [len(el.perimeter()) for el in femesh.element_iterator()]
    #RCL: One gets this for allEConSigs: [[elnnodes0,elnnodes1,...],[elnnodes0',elnnodes1',...],...]
    allEConSigs = mpitools.Allgather_IntVec(myEConSigs, size_known=nelems)

    # element connectivity
    #RCL: nodedict must be a map to the 0-based indices of the nodes
    myECons = [ [nodedict[nd.index()] for nd in el.node_iterator()]
                for el in femesh.element_iterator()]
    myECons = reduce(lambda x,y: x+y, myECons)

    #RCL: conSizes looks like [[elnnodes0+elnnodes1+...],[elnnodes0'+elnnodes1'+...],...]
    conSizes = [reduce(lambda x,y: x+y, aECS) for aECS in allEConSigs]

    #RCL: temp looks like
    # [[el0_nodeindex1,el0_nodeindex2,...el1_nodeindex1,el1_nodeindex2,...],[el0'_nodeindex1,el0'_nodeindex2,...,el1'_nodeindex1,el1'_nodeindex2,...],...]
    temp = mpitools.Allgather_IntVec(myECons, size_known=conSizes)
    
    #RCL: The connectivity information could still be shrunk, but its hard...
    # If we store the nodes for each element, duplicate nodes results. Storage becomes ~4*(number of (double x and double y))

    def listrize(list, signature):
        #RCL: nsig is the number of elements if allEConsigs[i] is passed as signature
        nsig = len(signature)
        count = 0
        output = [[] for i in range(nsig)]
        for i in range(nsig):
            for j in range(signature[i]):
                output[i].append(list[count])
                count += 1
        return output

    #RCL: allECons looks like [[[nodeindex1,nodeindex2,...],[nodeindex1',nodeindex2',...],...], [], ...]
    allECons = [listrize(temp[i], allEConSigs[i]) for i in range(_size)]
    if _rank == 0:
        femesh.all_meshskeletons = {"nodes": allCoords,
                              "elements": allECons}

#####################################

def renameMesh_parallel(menuitem, mesh, name):
    oldmeshpath = labeltree.makePath(mesh)
    themesh = oof2.engine.mesh.meshes[oldmeshpath]
    themesh.reserve()
    themesh.begin_writing()
    try:
        themesh.rename(name, exclude=oldmeshpath[-1])
    finally:
        themesh.end_writing()
        themesh.cancel_reservation()

ipcmeshmenu.addItem(oofmenu.OOFMenuItem(
    'Rename',
    callback=renameMesh_parallel,
    threadable=oofmenu.PARALLEL_THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('name', '', tip='New name for the mesh.')
            ]
    ))

#######################################

def deleteMesh_parallel(menuitem, mesh):
    meshobj = oof2.engine.mesh.meshes[mesh]
    ## Since the mesh is being destroyed, we have to acquire its lock,
    ## but we *don't* release the lock.  Nobody else should be doing
    ## anything to the mesh, ever. 
    meshobj.reserve()
    meshobj.begin_writing()
    try:
        meshobj.destroy()         # removes mesh from oof2.engine.mesh.meshes
    finally:
        meshobj.end_writing()
        meshobj.cancel_reservation()

ipcmeshmenu.addItem(oofmenu.OOFMenuItem(
    'Delete',
    threadable=oofmenu.PARALLEL_THREADABLE,
    callback=deleteMesh_parallel,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString)
    ]
    ))

#######################################

def copyMesh_parallel(menuitem, mesh, name,
             copy_field, copy_equation, copy_bc):
    notifications = []
    basemesh = oof2.engine.mesh.meshes[mesh]
    basemesh.begin_reading()
    try:
        edict = basemesh.elementdict
        copiedmeshname = name

        skel = basemesh.getSkeleton()
        skelpath = labeltree.makePath(basemesh.path())[:-1]

        copiedfemesh = skel.femesh_shares(edict, basemesh.materialfactory)
        newmesh = oof2.engine.mesh.meshes.add(
            skelpath+[copiedmeshname],
            copiedfemesh,
            parent=skeletoncontext.skeletonContexts[skelpath],
            skeleton=skel, elementdict=edict,
            materialfactory=basemesh.materialfactory)
        newmesh.reserve()
        newmesh.begin_writing()
        try:
            copiedmesh = skelpath+[copiedmeshname]
            copiedmeshfullname = string.join(copiedmesh,":")
            basefemesh = basemesh.femesh()

            if copy_field:
                for field in basemesh.all_fields():
                    copiedfemesh.define_field(field)
                    notifications.append(
                        ("field defined", copiedmeshfullname,
                         field.name(), 1) )
                    copiedfemesh.acquire_field_data(field, basefemesh)
                    if basefemesh.is_active_field(field):
                        copiedfemesh.activate_field(field)
                        notifications.append(
                        ("field activated",
                         copiedmeshfullname, field.name(), 1) )
                    if basefemesh.in_plane(field):
                        field.set_in_plane(copiedfemesh, 1)
                        notifications.append(
                            ("field inplane",
                             copiedmeshfullname, field.name(), 1) )

            if copy_equation:
                for eqn in basemesh.all_equations():
                    copiedfemesh.activate_equation(eqn)
                    notifications.append(
                        ('equation activated',
                         copiedmeshfullname, eqn.name(), 1) )

            if copy_bc:
                for (bcname, bc) in basemesh.allBoundaryConds():
                    copied = bc.copy(bc.boundary)
                    copied.add_to_mesh(bcname, copiedmesh)
        finally:
            newmesh.end_writing()
            newmesh.cancel_reservation()
    finally:
        basemesh.end_reading()

    for n in notifications:
        switchboard.notify(*n)


ipcmeshmenu.addItem(oofmenu.OOFMenuItem(
    'Copy', callback=copyMesh_parallel,
    threadable=oofmenu.PARALLEL_THREADABLE,
    params= parameter.ParameterGroup(
    whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                          tip=parameter.emptyTipString),
    parameter.StringParameter('name'),
    parameter.BooleanParameter('copy_field', value=1, tip='Copy fields?'),
    parameter.BooleanParameter('copy_equation', value=1, tip='Copy equation?'),
    parameter.BooleanParameter('copy_bc', value=1,
                               tip='Copy boundary conditions?') )
    ))


########################################################################
# Field (Define/Undefine, Activate/Deactivate, In_Plane/Out_of_Plane)
########################################################################

## OOF.LoadData.IPC.Field
ipcfieldmenu = parallelmainmenu.ipcmenu.addItem(
    oofmenu.OOFMenuItem('Field', secret=1, no_log=1)
    )

#TODO: Mimick the other menus in meshmenu.py. Add initField? Check if buttons/checkboxes are active for the background processes?

def parallel_defineField(menuitem, mesh, field):
    # mesh and field are names, not objects
    meshcontext = oof2.engine.mesh.meshes[mesh]
    meshcontext.reserve()
    meshcontext.begin_writing()
    try:
        themesh = meshcontext.femesh()
        themesh.define_field(field)
    finally:
        meshcontext.end_writing()
        meshcontext.cancel_reservation()

    switchboard.notify("field defined", mesh, field.name(), 1)
    meshcontext.autoenableBCs()
    oof2.engine.mesh.meshes[mesh].changed()

def parallel_undefineField(menuitem, mesh, field):
    meshcontext = oof2.engine.mesh.meshes[mesh]
    meshcontext.reserve()
    meshcontext.begin_writing()
    try:
        meshcontext.femesh().undefine_field(field)
    finally:
        meshcontext.end_writing()
        meshcontext.cancel_reservation()

    switchboard.notify("field defined", mesh, field.name(), 0)
    meshcontext.autoenableBCs()
    oof2.engine.mesh.meshes[mesh].changed()

ipcfieldmenu.addItem(oofmenu.OOFMenuItem(
    'Define',
    threadable=oofmenu.PARALLEL_THREADABLE,
    callback=parallel_defineField,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
    ]
    ))

ipcfieldmenu.addItem(oofmenu.OOFMenuItem(
    'Undefine',
    threadable=oofmenu.PARALLEL_THREADABLE,
    callback=parallel_undefineField,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
            ]
    ))

def parallel_activateField(menuitem, mesh, field):
    activated = False
    meshcontext = oof2.engine.mesh.meshes[mesh]
    meshcontext.reserve()
    meshcontext.begin_writing()
    try:
        themesh = meshcontext.femesh()
        if themesh.is_defined_field(field):
            themesh.activate_field(field)
            activated = True
        else:
            reporter.report(
                "You must define a Field before you can activate it.")
    finally:
        meshcontext.end_writing()
        meshcontext.cancel_reservation()
    if activated:
        switchboard.notify("field activated", mesh, field.name(), 1)
        meshcontext.autoenableBCs()
    oof2.engine.mesh.meshes[mesh].changed()
    
def parallel_deactivateField(menuitem, mesh, field):
    deactivated = False
    meshcontext = oof2.engine.mesh.meshes[mesh]
    meshcontext.reserve()
    meshcontext.begin_writing()
    try:
        themesh = meshcontext.femesh()
        if themesh.is_active_field(field):
            themesh.deactivate_field(field)
            deactivated = True
        else:
            reporter.report(
                "You must define and activate a Field before you can deactivate it.")
    finally:
        meshcontext.end_writing()
        meshcontext.cancel_reservation()
    if deactivated:
        switchboard.notify("field activated", mesh, field.name(), 0)
        meshcontext.autoenableBCs()
    oof2.engine.mesh.meshes[mesh].changed()

ipcfieldmenu.addItem(oofmenu.OOFMenuItem(
    "Activate",
    threadable=oofmenu.PARALLEL_THREADABLE,
    callback=parallel_activateField,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
    ]
    ))

ipcfieldmenu.addItem(oofmenu.OOFMenuItem(
    'Deactivate',
    threadable=oofmenu.PARALLEL_THREADABLE,
    callback=parallel_deactivateField,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
            ]
    ))

def parallel_inPlaneField(menuitem, mesh, field):
    meshcontext = oof2.engine.mesh.meshes[mesh]
    meshcontext.reserve()
    meshcontext.begin_writing()
    try:
        themesh = meshcontext.femesh()
        field.set_in_plane(themesh, 1)
    finally:
        meshcontext.end_writing()
        meshcontext.cancel_reservation()

    switchboard.notify("field inplane", mesh, field.name(), 1)
    oof2.engine.mesh.meshes[mesh].changed()
    
def parallel_outOfPlaneField(menuitem, mesh, field):
    meshcontext = oof2.engine.mesh.meshes[mesh]
    meshcontext.reserve()
    meshcontext.begin_writing()
    try:
        themesh = meshcontext.femesh()
        field.set_in_plane(themesh, 0)
    finally:
        meshcontext.end_writing()
        meshcontext.cancel_reservation()
    switchboard.notify("field inplane", mesh, field.name(), 0)    
    oof2.engine.mesh.meshes[mesh].changed()

ipcfieldmenu.addItem(oofmenu.OOFMenuItem(
    'In_Plane',
    threadable=oofmenu.PARALLEL_THREADABLE,
    callback=parallel_inPlaneField,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
            ]
    ))

ipcfieldmenu.addItem(oofmenu.OOFMenuItem(
    'Out_of_Plane',
    threadable=oofmenu.PARALLEL_THREADABLE,
    callback=parallel_outOfPlaneField,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            meshparameters.FieldParameter('field', tip=parameter.emptyTipString)
            ]
    ))

#####################################################################
# Equation (Activate/Deactivate)
#####################################################################

## OOF.LoadData.IPC.Equation
ipceqnmenu = parallelmainmenu.ipcmenu.addItem(
    oofmenu.OOFMenuItem('Equation', secret=1, no_log=1)
    )

def parallel_activateEquation(menuitem, mesh, equation):
    meshcontext = oof2.engine.mesh.meshes[mesh]
    meshcontext.reserve()
    meshcontext.begin_writing()
    try:
        themesh = meshcontext.femesh()
        themesh.activate_equation(equation)
    finally:
        meshcontext.end_writing()
        meshcontext.cancel_reservation()
        
    switchboard.notify('equation activated', mesh, equation.name(), 1)
    meshcontext.autoenableBCs()
    oof2.engine.mesh.meshes[mesh].changed()
    
def parallel_deactivateEquation(menuitem, mesh, equation):
    meshcontext = oof2.engine.mesh.meshes[mesh]
    meshcontext.reserve()
    meshcontext.begin_writing()
    try:
        themesh = meshcontext.femesh()
        themesh.deactivate_equation(equation)
    finally:
        meshcontext.end_writing()
        meshcontext.cancel_reservation()

    switchboard.notify('equation activated', mesh, equation.name(), 0)
    meshcontext.autoenableBCs()
    oof2.engine.mesh.meshes[mesh].changed()

ipceqnmenu.addItem(oofmenu.OOFMenuItem(
    'Activate',
    callback=parallel_activateEquation,
    threadable=oofmenu.PARALLEL_THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            meshparameters.EquationParameter('equation',
                                             tip=parameter.emptyTipString)
    ]
    ))

ipceqnmenu.addItem(oofmenu.OOFMenuItem(
    'Deactivate',
    callback=parallel_deactivateEquation,
    threadable=oofmenu.PARALLEL_THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            meshparameters.EquationParameter('equation',
                                             tip=parameter.emptyTipString)
    ]
    ))


##########################################################

# Copy the field state (definitions, active-ness, planarity) of one
# mesh into another.
def parallel_copyFieldState(menuitem, source, target):
    notifications = []
    if source == target:
        return
    source_mesh = oof2.engine.mesh.meshes[source]
    target_mesh = oof2.engine.mesh.meshes[target]
    source_mesh.begin_reading()
    target_mesh.reserve()
    target_mesh.begin_writing()
    try:
        source_obj = source_mesh.getObject()
        target_obj = target_mesh.getObject()
        source_fields = source_obj.all_fields()
        target_fields = target_obj.all_fields()

        if source_obj is target_obj:
            raise ooferror.ErrUserError('Source and target must differ!')

        # Undefine all the fields in the target that are not in the source.
        for f in target_fields:
            if not source_obj.is_defined_field(f):
                target_obj.undefine_field(f)
                notifications.append( ("field defined", target, f.name(), 0) )

        for f in source_fields:
            # Definition.
            if not target_obj.is_defined_field(f):
                target_obj.define_field(f)
                notifications.append( ("field defined", target, f.name(), 1) )

            # Activation.
            if source_obj.is_active_field(f):
                if not target_obj.is_active_field(f):
                    target_obj.activate_field(f)
                    notifications.append(
                        ("field activated", target, f.name(), 1) )
            else:
                if target_obj.is_active_field(f):
                    target_obj.deactivate_field(f)
                    notifications.append(
                        ("field activated", target, f.name(), 0) )

            # Planarity.
            if source_obj.in_plane(f):
                if not target_obj.in_plane(f):
                    f.set_in_plane(target_obj, 1)
                    notifications.append( 
                        ("field inplane", target, f.name(), 1) )
            else:
                if target_obj.in_plane(f):
                    f.set_in_plane(target_obj, 0)
                    notifications.append( 
                        ("field inplane", target, f.name(), 0) )

            try:
                initializer = source_mesh.initializers[f]
            except KeyError:
                pass
            else:
                target_mesh.initialize_field(f, initializer)
                notifications.append( ("field initialized") )
    finally:
        source_mesh.end_reading()
        target_mesh.end_writing()
        target_mesh.cancel_reservation()

    # Make all the switchboard notifications outside the locked region.
    for n in notifications:
        switchboard.notify(*n)

    # Update BCs in both meshes.
    target_mesh.autoenableBCs()

    target_mesh.changed()
    switchboard.notify("redraw")
        


ipcmeshmenu.addItem(oofmenu.OOFMenuItem(
    'Copy_Field_State',
    callback=parallel_copyFieldState,
    threadable=oofmenu.PARALLEL_THREADABLE,
    params=[whoville.WhoParameter('source',oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            whoville.WhoParameter('target',oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString)]
                        ) )




# Likewise for equation state.

def parallel_copyEquationState(menuitem, source, target):
    notifications = []
    if source == target:
        return
    source_mesh = oof2.engine.mesh.meshes[source]
    target_mesh = oof2.engine.mesh.meshes[target]
    source_mesh.begin_reading()
    target_mesh.reserve()
    target_mesh.begin_writing()
    try:
        source_obj = source_mesh.getObject()
        target_obj = target_mesh.getObject()
        source_eqns = source_obj.all_equations()
        target_eqns = target_obj.all_equations()

        for e in target_eqns:
            if not source_obj.is_active_equation(e):
                target_obj.deactivate_equation(e)
                notifications.append(
                    ("equation activated", target, e.name(), 0) )
        for e in source_eqns:
            if not target_obj.is_active_equation(e):
                target_obj.activate_equation(e)
                notifications.append(
                        ("equation activated", target, e.name(), 1) )
    finally:
        source_mesh.end_reading()
        target_mesh.end_writing()
        target_mesh.cancel_reservation()

    for n in notifications:
        switchboard.notify(*n)

    target_mesh.autoenableBCs()
    target_mesh.changed()


ipcmeshmenu.addItem(oofmenu.OOFMenuItem(
    'Copy_Equation_State',
    callback=parallel_copyEquationState,
    threadable=oofmenu.PARALLEL_THREADABLE,
    params=[whoville.WhoParameter('source',oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            whoville.WhoParameter('target',oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString)]
    ) )
