# -*- python -*-
# $RCSfile: guitests.py,v $
# $Revision: 1.12 $
# $Author: reida $
# $Date: 2006/09/07 13:04:06 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# This file looks for all subdirectories of the current directory and
# runs the gui test contained in each one.  The tests are run in
# alphabetical order of the subdirectory name.  It is assumed that
# each subdirectory contains a file named "log.py".  The test is run by
# executing
#         oof2 --pathdir <subdirectory> --replay <subdirectory>/log.py
# and testing its return value. The subdirectory is added to the
# python path so that the log file can contain import statements that
# load tests from other files in the subdirectory.

# The subdirectory can contain a flie called "args" which contains a
# single line of arguments to be added to the oof2 command.  It can
# also contain a file named 'cleanup.py' which will be run after the
# test, if the test is successful.  cleanup.py is run in the
# guitests.py environment.

import os
import sys
import string
import getopt

delaystr = None
debug = False

def run_tests(dirs, rerecord):
    for dir in dirs:
        if not os.path.isdir(dir):
            print >> sys.stderr, "Can't find directory", dir
            return
        if os.path.exists(os.path.join(dir, 'args')):
            argfile = open(os.path.join(dir, 'args'))
            extraargs = argfile.readline().rstrip()
            argfile.close()
        else:
            extraargs = ""
        global delaystr
        if delaystr:
            extraargs += " --replaydelay=" + delaystr
        if debug:
            extraargs += " --debug"

        if rerecord:
            replayarg = 'rerecord'
        else:
            replayarg = 'replay'
        cmd = "oof2 --pathdir . --pathdir %s --%s %s %s" \
              % (dir, replayarg, os.path.join(dir, "log.py"), extraargs)
        print >> sys.stderr, "-------------------------"
        print >> sys.stderr, "--- Running %s" % cmd
        os.putenv('OOFTESTDIR', dir)
        result = os.system(cmd)
        if result != 0:
            print "Test", dir, "failed!"
            sys.exit(1)
        print >> sys.stderr, "--- Finished %s" % dir

        cleanupscript = os.path.join(dir, 'cleanup.py')
        if os.path.exists(cleanupscript):
            execfile(cleanupscript)
          
    print >> sys.stderr, "All tests ran successfully!"

excluded = ['CVS','TEST_DATA']

def get_dirs():
    files = [f for f in os.listdir('.')
             if os.path.isdir(f) and f not in excluded]
    files.sort()
    return files

def removefile(filename):
    if os.path.exists(filename):
        os.remove(filename)
    
if __name__ == '__main__':
    try:
        optlist, args = getopt.getopt(sys.argv[1:], '', ['delay=', 'debug',
                                                         'from=', 'to=',
                                                         'rerecord'])
    except getopt.error, message:
        print message
        sys.exit(1)
    fromdir = None
    todir = None
    rerecord = False
    for opt in optlist:
        if opt[0] == "--debug":
            debug = True
        elif opt[0] == "--delay":
            delaystr = opt[1]
        elif opt[0] == '--from':
            # normpath is necessary here because if the shell's
            # filename completion was used to construct the argument,
            # the directory name may have a trailing slash, and the
            # index() calls below will fail.
            fromdir = os.path.normpath(opt[1])
        elif opt[0] == '--to':
            todir = os.path.normpath(opt[1])
        elif opt[0] == '--rerecord':
            rerecord = True
    if args:
        run_tests(args, rerecord)
    else:
        dirs = get_dirs()
        if fromdir and not todir:
            start = dirs.index(fromdir)
            run_tests(dirs[start:], rerecord)
        elif todir and not fromdir:
            end = dirs.index(todir)
            run_tests(dirs[:end+1], rerecord)
        elif todir and fromdir:
            start = dirs.index(fromdir)
            end = dirs.index(todir)
            run_tests(dirs[start:end+1], rerecord)
        else:
            run_tests(dirs, rerecord)
                         
